﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Windows.Media.Imaging;

namespace HeadTraxExtreme
{
    /// <summary>
    /// Class that represents an employee.
    /// </summary>
    [SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable", Justification = "Not relevant for intended scenarios.")]
    public class Employee : INotifyPropertyChanged
    {
        public static int TotalEmployees { get; set; }
        [Display(Name = "Alias")]
        public string Alias { get; private set; }
        [Display(Name = "Manager Alias")]
        public string ManagerAlias { get; private set; }
        [Display(AutoGenerateField = false)]
        public string FullName
        {
            get { return _fullName; }
            set
            {
                // Parse full name into first/last names
                _fullName = value;
                var commaIndex = FullName.IndexOf(',');
                if (-1 != commaIndex)
                {
                    LastName = FullName.Substring(0, commaIndex);
                    FirstName = FullName.Substring(commaIndex + 1).Trim().Split()[0];
                }
                else
                {
                    LastName = FullName;
                    FirstName = "";
                }
            }
        }
        private string _fullName;
        [Display(Name = "Last Name")]
        public string LastName { get; private set; }
        [Display(Name = "First Name")]
        public string FirstName { get; private set; }
        [Display(AutoGenerateField = false)]
        public Employee Manager { get; set; }
        [Display(AutoGenerateField = false)]
        public IList<Employee> DirectReports { get; private set; }
        [Display(Name = "Direct Reports")]
        public int DirectReportsCount
        {
            get { return DirectReports.Count; }
        }
        [Display(AutoGenerateField = false)]
        public IEnumerable<Employee> AllReports
        {
            get
            {
                // Return all reports by walking everyone's direct reports
                foreach (var directReport in DirectReports)
                {
                    yield return directReport;
                    foreach (var report in directReport.AllReports)
                    {
                        yield return report;
                    }
                }
            }
        }
        [Display(Name = "All Reports")]
        public int AllReportsCount
        {
            get { return AllReports.Count(); }
        }
        [Display(Name = "Title")]
        public string Title { get; set; }
        [Display(Name = "Cost Center")]
        public string CostCenter { get; set; }
        [Display(Name = "Office")]
        public string Office { get; set; }
        [Display(Name = "Starting Date")]
        public DateTime StartingDate { get; set; }
        [Display(Name = "\"BillG\" Number")]
        public int BillGNumber
        {
            get
            {
                // Calculate the number of levels down from the top-most leader
                int number = -1;
                var manager = Manager;
                while (null != manager)
                {
                    number++;
                    manager = manager.Manager;
                }
                return number;
            }
        }
        [Display(AutoGenerateField = false)]
        public bool IsExpanded
        {
            get { return _isExpanded; }
            set
            {
                _isExpanded = value;
                InvokePropertyChanged("IsExpanded");
            }
        }
        private bool _isExpanded;
        [Display(AutoGenerateField = false)]
        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                _isSelected = value;
                InvokePropertyChanged("IsSelected");
            }
        }
        private bool _isSelected;
        [Display(AutoGenerateField = false)]
        public string SearchableForm
        {
            get { return string.Concat(FullName, " [", Alias, "]"); }
        }
        public BitmapImage BitmapImage
        {
            get
            {
                if (null == _bitmapImage)
                {
                    // Create a default image
                    _bitmapImage = new BitmapImage();
                    _bitmapImage.SetSource(Assembly.GetExecutingAssembly().GetManifestResourceStream("HeadTraxExtreme.MissingImage.png"));

                    // Make a web request for the actual image
                    var client = new WebClient();
                    client.OpenReadCompleted += delegate(object s, OpenReadCompletedEventArgs e)
                    {
                        // Handle completion by updating the image (if successful)
                        if (null == e.Error)
                        {
                            _bitmapImage.SetSource((Stream)e.Result);
                        }
                    };

                    // Start the web request
                    client.OpenReadAsync(new Uri(App.Current.Host.Source, "../Images/" + Alias + ".jpg"));
                }
                return _bitmapImage;
            }
        }
        private BitmapImage _bitmapImage;

        // Construct a new instance
        public Employee(string alias, string managerAlias)
        {
            Alias = alias;
            ManagerAlias = managerAlias;
            DirectReports = new ObservableCollection<Employee>();
        }

        // Implement INotifyPropertyChanged
        private void InvokePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (null != handler)
            {
                try
                {
                    handler.Invoke(this, new PropertyChangedEventArgs(propertyName));
                }
                catch (NullReferenceException)
                {
                    // Something subscribed to this event dereferences null every now and then
                }
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
