﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Xml.Linq;

/*
 * NOTES:
 * Command to run to create data file from raw source:
 *   LogParser.exe -i:CSV -o:XML -compact:ON "SELECT FullName, Email, AddressBookTitle, ReportsToEmail, CostCenterName, ServiceAwardDate, Building, Room into HeadTraxData.xml from HeadTraxData.csv"
 */

namespace HeadTraxExtreme
{
    /// <summary>
    /// Class that implements the main functionality.
    /// </summary>
    public partial class MainPage : UserControl
    {
        private Employee _root;
        private PagedCollectionView _pagedCollectionView;

        // Constructs a new instance
        public MainPage()
        {
            // Initialize
            InitializeComponent();

            // Read and parse the XML employee data
            _root = new Employee("", null);
            var employees = new Dictionary<string, Employee>();
            employees.Add(_root.Alias, _root);
            using (var stream = Assembly.GetExecutingAssembly().GetManifestResourceStream("HeadTraxExtreme.HeadTraxData.xml"))
            {
                var data = XDocument.Load(stream);
                foreach (var node in data.Element("ROOT").Elements("ROW"))
                {
                    // Create an Employee object for this node
                    var alias = node.Element("Email").Value;
                    var employee = new Employee(alias, node.Element("ReportsToEmail").Value);
                    employee.FullName = node.Element("FullName").Value;
                    employee.Title = node.Element("AddressBookTitle").Value;
                    employee.CostCenter = node.Element("CostCenterName").Value;
                    DateTime startingDate;
                    if (!DateTime.TryParse(node.Element("ServiceAwardDate").Value, out startingDate))
                    {
                        startingDate = DateTime.Now;
                    }
                    employee.StartingDate = startingDate.Date;
                    employee.Office = node.Element("Building").Value + "/" + node.Element("Room").Value;
                    employees.Add(employee.Alias, employee);
                }
            }

            // Match up employees with their managers
            foreach (var employee in employees.Values.Where(e => null != e.ManagerAlias))
            {
                var manager = employees[employee.ManagerAlias];
                employee.Manager = manager;
                manager.DirectReports.Add(employee);
            }

            // Expand the top-level
            foreach (var employee in _root.DirectReports)
            {
                employee.IsExpanded = true;
            }

            // Store results
            Employee.TotalEmployees = employees.Count - 1;
            DataContext = _root;
        }

        // Handle different kinds of selection changed notifications
        private void SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (null != e.NewValue)
            {
                Select((Employee)e.NewValue);
            }
        }
        private void SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if ((null != e.AddedItems) && (1 <= e.AddedItems.Count))
            {
                Select(e.AddedItems.Cast<Employee>().First());
            }
        }
        private void CurrentItemChanged(object sender, EventArgs e)
        {
            Select((Employee)Form.CurrentItem);
        }

        // Selects a new employee
        private void Select(Employee employee)
        {
            // Select the new employee everywhere
            Box.SelectedItem = employee;
            Grid.SelectedItem = employee;
            Dispatcher.BeginInvoke(() => Grid.ScrollIntoView(employee, null));
            Form.CurrentItem = employee;
            employee.IsSelected = true;

            // Expand the employee's management chain
            var manager = employee.Manager;
            while (null != manager)
            {
                manager.IsExpanded = true;
                manager = manager.Manager;
            }
        }

        private void Grouping_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Abort if called before initialization
            if (null == _root)
            {
                return;
            }

            // Create a PagedCollectionView now that it's necessary
            if (null == _pagedCollectionView)
            {
                _pagedCollectionView = new PagedCollectionView(_root.AllReports);
                Grid.ItemsSource = _pagedCollectionView;
            }

            // Clear the view
            _pagedCollectionView.GroupDescriptions.Clear();

            // Set the new group description
            if ((null != e.AddedItems) && (1 <= e.AddedItems.Count))
            {
                var groupInfo = (GroupInfo)e.AddedItems[0];
                if (null != groupInfo.PropertyName)
                {
                    _pagedCollectionView.GroupDescriptions.Add(new PropertyGroupDescription(groupInfo.PropertyName));
                }
            }
        }

        // Implement a workaround for the Silverlight XAML parser limitation that
        // prevents the following from working:
        //   <Setter Property="IsSelected" Value="{Binding IsSelected}"/>
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters", Justification = "Specifically for TreeViewItems.")]
        public static bool GetPrepareTreeViewItem(TreeViewItem element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }
            return (bool)element.GetValue(PrepareTreeViewItemProperty);
        }
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters", Justification = "Specifically for TreeViewItems.")]
        public static void SetPrepareTreeViewItem(TreeViewItem element, bool value)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }
            element.SetValue(PrepareTreeViewItemProperty, value);
        }
        public static readonly DependencyProperty PrepareTreeViewItemProperty =
            DependencyProperty.RegisterAttached(
                "PrepareTreeViewItem",
                typeof(bool),
                typeof(MainPage),
                new PropertyMetadata(false, OnPrepareTreeViewItemPropertyChanged));
        private static void OnPrepareTreeViewItemPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var treeViewItem = (TreeViewItem)d;
            treeViewItem.SetBinding(TreeViewItem.IsExpandedProperty, IsExpandedBinding);
            treeViewItem.SetBinding(TreeViewItem.IsSelectedProperty, IsSelectedBinding);
        }
        private static Binding IsExpandedBinding = new Binding { Path = new PropertyPath("IsExpanded"), Mode = BindingMode.TwoWay };
        private static Binding IsSelectedBinding = new Binding { Path = new PropertyPath("IsSelected"), Mode = BindingMode.TwoWay };
    }
}
